﻿using System;
using System.Drawing;

using State.Events;
using State.Armor;
using State.Shooting;
using State.Units.States;
using State.FactionInfo;

namespace State.Units
{
    public abstract class GameUnit
    {
        private Point _position;
        private IUnitState _state;

        public event EventHandler<PositionChangedEventArgs> PositionChanged;

        public UnitSide Side { get; set; /* Can set, a unit can switch sides */ }
        public UnitBehaviour Behaviour { get; set; }
        public string Description { get; set; }

        public Point Position
        {
            get
            {
                return _position;
            }
            set
            {
                _position = value;
                OnPositionChanged();
            }
        }

        public UnitType Type { get; protected set; }
        public IArmorBehaviour Armor { get; protected set; }
        public IShootingBehaviour ShootingBehaviour { get; protected set; }

        protected FactionDetailsFactory FactionDetails { get; private set; }

        // Unit classes each take care of their own Display
        public abstract void Display();

        public GameUnit(FactionDetailsFactory factionFactory)
        {
            _state = GetNormalState();

            FactionDetails = factionFactory;

            Side = factionFactory.GetSide();
            Behaviour = factionFactory.GetBehaviour();
            Description = "Undefined Game Unit";
        }

        protected void OnPositionChanged()
        {
            if (PositionChanged != null)
            {
                PositionChanged(this, new PositionChangedEventArgs(Position));
            }
        }

        public void SetState(IUnitState state)
        {
            _state = state;
        }

        public IUnitState GetDeadState()
        {
            return new DeadState();
        }

        public IUnitState GetNormalState()
        {
            return new NormalState(this);
        }

        public void PerformShoot()
        {
            _state.PerformShoot();
        }

        public void Move(Point newLocation)
        {
            _state.Move(newLocation);
        }

        public void Kill()
        {
            // Kill the unit.
            _state.Kill();
        }

        public override string ToString()
        {
            return string.Format("{0}, {1}", Type, _state);
        }
    }
}
